#
# animobject.py
#
#
# Definition of animobject class 
#
# 1.8.2005 Mazany Ondrej - email:mazany.o@seznam.cz
#
# This class is used for storing information about objects in scene which will be animated.
# It has three main attributes - name - name of object in Blender scene
#                                Position and Rotation
# The attribs attribute contains list of attributes of object which should be changed, like positon
# in x coord (px) or rotation around z axis (rz)                             
#

# importing required modules
import Blender 
from Blender.Object import *
from bone import *

class AnimObject:
    "A AnimObject class which is used by animator project for animated objects configuration"
    def __init__(self):
        self.name = ""                           # name of object
        self.attribs = list()                    # attributes of this object to be setted up
        self.Position = [ 0.0, 0.0, 0.0]         # position of this object
        self.Rotation = [ 0.0, 0.0, 0.0]         # rotation of this object
        self.Size = [ 1.0, 1.0, 1.0 ]            # size of object
        self.Bones = list()                      # the list of Bones
        self.ob = None                           # blenders object
    def setUp(self): # configures object
        "Sets the object up in the current frame and also its bones if avaible"
        try:
            self.ob = Blender.Object.Get(self.name)    # find the object in the scene
            for at in self.attribs: # check which attributes should be changed
                if at == "px":
                    self.ob.LocX = self.Position[0]
                if at == "py":
                    self.ob.LocY = self.Position[1]
                if at == "pz":
                    self.ob.LocZ = self.Position[2]
                if at == "rx":
                    self.ob.RotX = self.Rotation[0]
                if at == "ry":
                    self.ob.RotY = self.Rotation[1]
                if at == "rz":
                    self.ob.RotZ = self.Rotation[2]
                if at == "sx":
                    self.ob.SizeX = self.Size[0]
                if at == "sy":
                    self.ob.SizeY = self.Size[1]
                if at == "sz":
                    self.ob.SizeZ = self.Size[2]
            try:
                self.ob.insertIpoKey( LOCROTSIZE) # insert IPO key in Blender for animation
            except:                               # this tell Blender to remember objects position
                print "Unable to set IPO!"        # and rotation and size in current frame
            print "Object",self.name,"configured : pos=",self.Position,";rot=",self.Rotation,";siz=",self.Size,"."
        except:
            print "!! Object",self.name,"not found in the scene !!"
        # try to set the bones up
        try:
            for b in self.Bones:
                if b <> None: b.setUp(self.ob)
                print "Bone",b.name,self.name,"'s object set up."
        except:
            print "!! Exception while trying to set the bones up for",self.name,"object!!"