#A Script to export mesh data during the animation


# importing modules
import Blender  # Blender API import
from Blender import *
from Blender import Object, NMesh, Mesh, Types
from rahat.bfunc import *
# global variable
global DEBUG
DEBUG = True
def exportVertices( object, outputdir ,endFrame=-1):
    """
    def exportVertices( object, outputdir ,endFrame=-1)
        In the output directory exports all the vertices of the mesh
        in the ascii format to the file begining with the name of the object 
        and with apendix _%framenumber.verts
        if endFrame==-1 then the final exported frame is obtained from 
        blenders end frame render settings
    """
    if DEBUG:
        print "exportVertices(", object,",", outputdir ,",",endFrame,")"
    if object.getType() != "Mesh" :
        print "error! def exportVertices( object, outputdir ,endFrame=-1): given object is not a Mesh type"
        print "       object type is: " + object.getType()
        return None    
    # go through all the frames
    scn = Blender.Scene.GetCurrent()
    context = scn.getRenderingContext()
    if endFrame == -1:
        lastf=context.endFrame()
    else:
        lastf=endFrame
    print "Exporting vertices.."
    for f in range(lastf):
        print "frame: ",f
        # set the filename
        filename=outputdir+"\\"+object.getName()+"_"+repr(f).zfill(5)+".verts"
        print "to:",filename
        Blender.Set( 'curframe', f)
        # Get the raw, deformed mesh
        mesh=getDeformedMeshData(object)
        # and the object matrix
        mat=object.getMatrix()        
        if object.getParent():
            mat = mat*object.getParent().getMatrix()
        # export the vertices
        try:
            file=open(filename,'w')
            for v in mesh.verts:
                # compute the world coordinates
                vco = Vector(v).resize4D()*mat
                file.write("%9.13f, %9.13f, %9.13f;\n" % (vco[0],vco[1],vco[2]))              
            file.close()
        except:
            print "Error during exporting vertices to file:",filename

def exportBones( object, outputdir ,endFrame=-1):
    """
    def exportVertices( object, outputdir ,endFrame=-1)
        In the output directory exports all the bones positions of the armature
        in the ascii format to the file begining with the name of the object 
        and with apendix _%framenumber.bones
        if endFrame==-1 then the final exported frame is obtained from 
        blenders end frame render settings
    """
    if DEBUG:
        print "exportVertices(", object,",", outputdir ,",",endFrame,")"
    if object.getType() != "Armature" :
        print "error! def exportVertices( object, outputdir ,endFrame=-1): given object is not a Armature type"
        print "       object type is: " + object.getType()
        return None    
    # go through all the frames
    scn = Blender.Scene.GetCurrent()
    context = scn.getRenderingContext()
    if endFrame == -1:
        lastf=context.endFrame()
    else:
        lastf=endFrame
    print "Exporting bones.."
    for f in range(lastf):
        print "frame: ",f
        # set the filename
        filename=outputdir+"\\"+object.getName()+"_"+repr(f).zfill(5)+".bones"
        print "to:",filename
        Blender.Set( 'curframe', f)
        # Get the raw, deformed mesh
        pose=object.getPose()
        # and the object matrix
        mat=object.getMatrix()
        if object.getParent():
            mat = mat*object.getParent().getMatrix()
        # export the vertices
        print "exporting"
        #try:
        file=open(filename,'w')
        print "opened"
        print pose.bones
        for bkey in pose.bones.keys():
            # compute the world coordinates 
            b=pose.bones[bkey]
            print b.name
            head=Vector(b.head).resize4D()*mat
            tail=Vector(b.tail).resize4D()*mat
            file.write(b.name+".head=[%9.13f, %9.13f, %9.13f]\n" % (head[0],head[1],head[2]))              
            file.write(b.name+".tail=[%9.13f, %9.13f, %9.13f]\n" % (tail[0],tail[1],tail[2]))              
        file.close()
        #except:
        #    print "Error during exporting bones positions to file:",filename
    

