from Blender import *
from rahat.calibim import *
from rahat.bfunc import *
from scipy import *
import numpy

# Class Calibrated Image
class FitObj:
    def __init__(self,BlenderObject,Silhouettes):
        """
        def __init__(BlenderObject,Silhouettes): inicializes class FitObj
        BlenderObject must be a NMesh Blender object
        Silhouettes is a list of CalibIm objects
        """
        self.sil = Silhouettes        
        self.obj = BlenderObject        
    def fitPosition(self):
        """
        def fitPosition(self): not implemented yet
        """
        return
    def fitMesh(self):
        """
        def fitMesh(): this fit a mesh to the silhouettes
        """
        #This should fit mesh into silhouette map
        updated = FALSE
        
        i=0 # silhouette index        
        mesh = getDeformedMeshData(self.obj)
        # We start with first only
        image = self.sil[i].cim.im
       
        # First Render Mesh to Silhouette Map
        # So we will change only vertices, where needed
        self.renderToMap()        
    
        # SOME INTELIGENT ALPHA PARAMETER COMPUTING SHOULD BE FOUND!!
        # Compute maximum vertex normal projection into image
        #maxalpha=0.00
        #for v in mesh.verts:               # loop the list of vertices             
            # Calculate normal projection
        #    pos = mat(v.no)
        #    pl = pos.tolist()                        
        #    pl = pl[0]
        #    pl.append(1.0)                
        #    pos = mat(pl,float)
        #    uv2 = self.sil[i].cim.P*pos.T;
        #    uc = uv2[0]/uv2[2]
        #    vc = uv2[1]/uv2[2]
        #    newalpha=sqrt(uc*uc+vc*vc)       
        #    if newalpha > maxalpha: 
        #        maxalpha=newalpha


        
        
        for v in mesh.verts:               # loop the list of vertices            
            # Calculate normal projection
            pos = mat(v.no)
            pl = pos.tolist()                        
            pl = pl[0]
            pl.append(1.0)                
            pos = mat(pl,float)
            uv2 = self.sil[i].cim.P*pos.T;
            uc = uv2[0]/uv2[2]
            vc = uv2[1]/uv2[2]
            alpha=sqrt(uc*uc+vc*vc)           
            #if alpha > 0.50*maxalpha: 
            #    print "Chance"
            #alpha=alpha/maxalpha*0.001 
            #if (alpha < 0.00001): alpha=0.00001           
            alpha=0.005
                # Default change
            pos = mat(self.obj.getLocation()) + mat(v.co) + alpha*mat(v.no)            
                # Project the new coord, need a homogenous coord
            pl = pos.tolist()                        
            pl = pl[0]
            pl.append(1.0)                
            pos = mat(pl,float)
            uv = self.sil[i].cim.P*pos.T;
                # recalculate x,y
            x = uv[0]/uv[2]
            y = uv[1]/uv[2]
            y = image.size[1] - y
                # if the vertex is not projected outside of the box
                # of silhouette and if it is projected on silhouette
            if ((x >= self.sil[i].left) & (x <= self.sil[i].right) &
                    (y >= self.sil[i].bottom) & (y <= self.sil[i].top)):                    
                    if (self.sil[i].silmap[int(x),int(y)] == 1):                      
                        loc = self.obj.getLocation()
                        v.co[0] = pos[0,0] - loc[0]
                        v.co[1] = pos[0,1] - loc[1]
                        v.co[2] = pos[0,2] - loc[2]    
                        updated = TRUE                                      
        # update on change
        mesh.update()
        return updated
    def renderToMap(self):
        # Render All Faces into Silhouette Map
        faces = getDeformedMeshData(self.obj).faces        
        for f in faces:
            coords=list()            
            for v in f.v:
                coord = self.projectVertex(v.co)
                coords.append(coord)
            self.sil[0].drawPolyToMap(coords,1)
    def projectVertex(self,vertex):
        pos = mat(self.obj.getLocation()) + mat(vertex)            
        # Project the new coord, need a homogenous coord
        pl = pos.tolist()                        
        pl = pl[0]
        pl.append(1.0)                
        pos = mat(pl,float)
        uv = self.sil[0].cim.P*pos.T;
        # recalculate x,y
        x = uv[0]/uv[2]
        y = uv[1]/uv[2]
        y = self.sil[0].cim.im.size[1] - y
        uv = [x,y]
        return uv

def adaptVerts( meshObject, images, meshColorIndex=1):
    """
    def adaptVerts( meshObject, images, meshColorIndex=1): return true if any
    of the vertices were changed. False if there was not any change.
    meshObject is Blender Object with linked Mesh Data Object.
    images is a list of CalibIm objects,where was the mesh rendered into.
    MeshColorIndex is the index of color of mesh in images
    """
    # vertices to change
    vertices = list()
    # get the mesh data
    #mesh = meshObject.getData(False,True)
    defmesh = getDeformedMeshData(meshObject)
    mesh=meshObject.getData()
    # get the bounding box
    bbox = meshObject.getBoundBox()
    # find the distance of the box and set the alpha
    v1 = bbox[0]
    alpha = 0
    for vec in bbox:
        v2 = vec-v1
        if abs(v2.length) > alpha:
            alpha = abs(v2.length)
    alpha = alpha / 10
    print "Alpha:", alpha
    # check all the vertices
    changed = False
    vindex=0
    for v in defmesh.verts:
        # get the vertex new location in the world in direction of its normal
        verco = Vector([v.co[0],v.co[1],v.co[2]])
        verno = Vector([v.no[0],v.no[1],v.no[2]])
        #print "---- processing vertex ------------------------"
        #print "vertex coordinates:",verco
        #print "vertex normal:",verno
        #print "change to:",(verco + verno*alpha)
        #print "projects to:",i.project( vertex[0], verco[1], vertex[2])
        vertex = meshObject.getMatrix() * (verco + verno*alpha).resize4D()
        # debug
        #print "in world: ", vertex
        # check the index at images
        outOfRange = False
        change = False
        for i in images:
            uv = i.project( vertex[0], vertex[1], vertex[2])
            #uv = i.getUVFromView( [vertex[0], vertex[1], vertex[2]])
            ind = i.getColIndexAt( uv[0], uv[1])
            rgba = indexToCol( int(ind))
            #print "projects to color: ",rgba," at: ",uv
            # if there is  a background
            if ((rgba[0] == 0) and 
                (rgba[1] == 0) and
                (rgba[2] == 0)): # and
                #(rgba[3] == 0)):
                #print "is out of range"
                outOfRange = True
            # if there is a something else then mesh
            if ind != meshColorIndex:
                #print "is not projected on mesh silhouette"
                change = True  
        # if it wasnt out of range, and should be changed, add it to list
        ver=mesh.verts[vindex]
        vindex=vindex+1
        if ((not outOfRange) and change):
            #vertices.append( [v, alpha])
            #print "Found: ",v.co,alpha,v.no 
            ver.co[0] = v.co[0] + v.no[0]*alpha
            ver.co[1] = v.co[1] + v.no[1]*alpha
            ver.co[2] = v.co[2] + v.no[2]*alpha
            #print "new vertex coords: ",v.co
            changed = True
    mesh.update()
    return changed
    
    

def fitMesh( meshObject, images, n=10):
    """
    def fitMesh( meshObject, images, n=10): fits a mesh to the silhouettes
    meshObject is the Blenders mesh object
    images is a list of images
    n is the maximum of repetitions
    This is old version
    """
    # get mesh data
    #mesh = getDeformedMeshData(meshObject)        
    # make copies of images
    silhouettes = list()
    for im in images:
        silhouettes.append( im.makeCopy())
    # render mesh to the images
    for sil in silhouettes:
        sil.renderToImage( meshObject)
    # repetitive search
    i=0
    # find vetices to adapt
    while adaptVerts( meshObject, images) and (i < n):
        i=i+1
    # end