#
# parseanim.py
#
# The main function is parseanim(filename), usage: parseanim("cameras.xml")
#
# 1.8.2005 Mazany Ondrej - email:mazany.o@seznam.cz
#
# Parse animation.xml file which contains configurations for cameras
# like their position and rotation. This module is a part of animator project for cmp@felk.cvut.cz.
#

# importing important modules
from frame import *
from animobject import *
from bone import *
from math import *
# global variables
global frames, current_element, renderpath, current_object;

# frames is a list of frame objects containing the frame data
frames = list()

# current object we work with
curobj = None

# function finds in the list item with attribute name
def findByName(objectlist,name):
    "This function returns object from objectlist which has the same name attribute"
    try:
        l = len(objectlist)
        i = 0;
        while (i < l):
            if (objectlist[i].name == name):
                       return objectlist[i]
            i = i + 1
        return None
    except:
        print "!! Error, the objects in list do not have attribute .name!!"

# function for parsing the object element
def object_element(name,attrs,objects):
    "This function is called to parse the object element and returns the parsed AnimObject"
    try:
        # find the object in the list
        obj = findByName(objects, attrs["name"])
        # if not found, add a new one
        if (obj == None):
            obj = AnimObject()
            objects.append( obj)                
        # set up a object
        for atrib in attrs:
            if atrib == "name":                    
                obj.name = attrs[ atrib] # setting the name
            if atrib == "px":
                obj.Position[0] = float( attrs[ atrib]) # setting the value
                obj.attribs.append("px")        # telling that this attribute has to be changed during animation
            if atrib == "py":
                obj.Position[1] = float( attrs[ atrib])
                obj.attribs.append("py")
            if atrib == "pz":
                obj.Position[2] = float( attrs[ atrib])
                obj.attribs.append("pz")                    
            if atrib == "rx":
                obj.Rotation[0] = radians(float( attrs[ atrib]))
                obj.attribs.append("rx")                   
            if atrib == "ry":
                obj.Rotation[1] = radians(float( attrs[ atrib]))
                obj.attribs.append("ry")                 
            if atrib == "rz":
                obj.Rotation[2] = radians(float( attrs[ atrib]))
                obj.attribs.append("rz")     
            if atrib == "sx":
                obj.Size[0] = float( attrs[ atrib])
                obj.attribs.append("sx")                   
            if atrib == "sy":
                obj.Size[1] = float( attrs[ atrib])
                obj.attribs.append("sy")                 
            if atrib == "sz":
                obj.Size[2] = float( attrs[ atrib])
                obj.attribs.append("sz")     
        return obj # return the parsed AnimObject
    except:
        print "!! Problem with setting the %s object !!" % ( attrs["name"])
        return None #problem with parsing, return None
        
# function for parsing bone element
def bone_element(name, attrs, bones):
    "This function parses the bone element"
    try:
        bn = findByName( bones, attrs["name"])
        # if not found add a new one
        if (bn == None):
            bn = Bone()
            bones.append( bn)            
        # we can set up a bone
        for atrib in attrs:
            if atrib == "name":
                bn.name = attrs[ atrib] # setting the name
            if atrib == "px":
                bn.Position[0] = float( attrs[ atrib]) # setting the value
                bn.attribs.append("px")        # telling that this attribute has to be changed during animation
            if atrib == "py":
                bn.Position[1] = float( attrs[ atrib])
                bn.attribs.append("py")
            if atrib == "pz":
                bn.Position[2] = float( attrs[ atrib])
                bn.attribs.append("pz")                    
            if atrib == "rx":
                bn.Rotation[0] = radians(float( attrs[ atrib]))
                bn.attribs.append("rx")                   
            if atrib == "ry":
                bn.Rotation[1] = radians(float( attrs[ atrib]))
                bn.attribs.append("ry")                 
            if atrib == "rz":
                bn.Rotation[2] = radians(float( attrs[ atrib]))
                bn.attribs.append("rz")               
    except:
        print "!! Problem with setting " + attrs["name"] + " bone"

# defining function for parsing the xml file
def start_element(name, attrs):
    "This function is automaticaly called by SAX parser when the start element is found"
    import Blender
    global frames,renderpath                         # frames list and render path 
    global curobj # the current AnimObject we work with
    lastframe = len(frames)-1 # the last frame
    
    if name == "renderpath":
        renderpath = attrs["path"]
        print "Render path :", renderpath

    if name == "startframe":  # set the start frame of animation
        scn = Blender.Scene.GetCurrent()
        context = scn.getRenderingContext()
        context.startFrame(int(attrs["n"]))

    if name == "endframe":        # set the end frame of animation
        scn = Blender.Scene.GetCurrent()
        context = scn.getRenderingContext()
        context.endFrame(int(attrs["n"]))

    if name == "frame":            # if found frame element, process it and add it to list
        print "Frame ", attrs["n"]
        newFrame = Frame()
        newFrame.n = int( attrs["n"] ) # the n attribute represents the number of frame
        frames.append( newFrame)    

    if name == "object":           # if found object element call the function
        print " - object :",attrs["name"]
        curobj = object_element(name,attrs,frames[lastframe].animobjects)

    if name == "bone": # if found bone element parse call the function
        print "     - bone: " + attrs["name"] + " related to: " + curobj.name 
        bone_element(name, attrs, curobj.Bones)        
        
# function when ending element found
def end_element(name):
    pass


# function for character data in xml file between start and end element        
def char_data(data):
    pass
   
# function called on the end of document     
def dummyfunc():
    pass
    
# This function parse the xml file and adds cameras into list of cameras
def parseanim(filename):
    "Parses animation file"
    # trying to parse the cameras.xml file
    try:
        import xml.parsers.expat               # try to import xml parser

        # creating a xml parser
        p = xml.parsers.expat.ParserCreate()

        # preparing the parser, setting handlers to functions
        p.StartElementHandler = start_element
        p.EndElementHandler = end_element
        p.CharacterDataHandler = char_data
        p.EndDoctypeDeclHandler = dummyfunc
        # opening the file
        try:
            f = open( filename, 'r')
        except:
            print "!! Can't open the %s file !!" % filename
        
        # parsing the file
        line = "line"
        print "Parsing the %s animation settings file:" % filename
        while line <> "":
            try:
                line = f.readline()                
                if (line <> ""): 
                    try:
                        #print line
                        p.Parse(line, False)
                    except:
                        print "!! XML Parsing error: "+xml.parsers.expat.ErrorString(p.ErrorCode)+" in "+line+" !!"
            except:
                print "!! Error during parsing the %s file !!" % filename
                line=""
        # closing the file
        try:
            f.close()
            print 'Animation file %s parsed!' % (filename)
            print "Results:"
            for fr in frames:
                for ao in fr.animobjects:
                    print "Frame:",fr.n,"object: ", ao.name
                    print "     attribs: ", ao.attribs,"\n     pos:",ao.Position,"\n     rot:",ao.Rotation
                    
        except:
            print "!! Prolem with closing the %s file !!" % filename
        
        
    # if something went wrong
    except:
        print '!! Problem with parsing %s animation config file!!' % (filename)
