#
# parsecam.py
#
# The main function is parsecam(filename), usage: parsecam("cameras.xml")
#
# 1.8.2005 Mazany Ondrej - email:mazany.o@seznam.cz
#
# Parse cameras.cam file which contains configurations for cameras
# like their position and rotation. This module is a part of animator project for cmp@felk.cvut.cz.
#
# file example:
## C=10.0,0.3,-0.0; # camera centre x,y,z
## f=1; # focal plane
## R=10.0,35.3,-60; # camera rotation around x,y,z
## k=4:3;    # ascpect ration u:v
##
## # rendering settings
## size=800x600 # resolution in pixels
## format=tga # output format
## ##    * aviraw: Uncompressed AVI files. AVI is a commonly used format on Windows plattforms
## ##    * avijpeg: AVI movie w/ Jpeg images
## ##    * avicodec: AVI using win32 codec
## ##    * quicktime: Quicktime movie (if enabled)
## ##    * tga: Targa files
## ##    * rawtga: Raw Targa files
## ##    * png: Png files
## ##    * bmp: Bitmap files
## ##    * jpg: Jpeg files
## ##    * hamx: Hamx files
## ##    * iris: Iris files
## ##    * iriz: Iris + z-buffer files
## ##    * ftype: Ftype file

# including camera class from camera.py
from rahat.calibcam import *

# import OS module and String module
import os
import string
from math import *

# global variables
global cameras, numcams

# defining the cameras array (list) containing the camera class object
cameras = []

# global variable of number of cameras
numcams = 0

# defining function for parsing the xml file
def start_element(name, attrs):
    global numcams                         # number of cameras readed
    global cameras                         # cameras list 
    if name == "camera":            # if found camera element, process it and add a camera to list
        #print "Camera ",numcams," name: ", attrs["name"]
        #print "pos=[",attrs[ "px"],",",attrs[ "py"],",",attrs[ "pz"],"] rot=[",attrs[ "rx"],",",attrs[ "ry"],",",attrs[ "rz"],"]"
        index = 0
        # looking for camera record in list
        if len(cameras) > 0:
            while (cameras[ index].name <> attrs[ "name"]) & (index < len( cameras)-1):
                index = index + 1
            # if not found, add a new one
            if cameras[ index].name <> attrs[ "name"]:
                cameras.append( CalibCam())
                index = len(cameras)-1
        else:
                cameras.append( CalibCam())
                index = len(cameras)-1            
        cameras[ index ].name = attrs[ "name"]             # and set it up
        cameras[ index ].setPosition( float( attrs[ "px"]), float( attrs[ "py"]), float( attrs[ "pz"]))
        cameras[ index ].setRotation( float(attrs[ "rx"]), float(attrs[ "ry"]), float(attrs[ "rz"]))
        
        
# function when ending element found
def end_element(name):
    global numcams
    if name == "camera":
        numcams = numcams + 1

# function for character data in xml file between start and end element        
def char_data(data):
    return True
        
# This function parse the xml.cam file and adds cameras into list of cameras
def parsecam(path):
    # list of cameras
    cameras = []
    # list files in directory
    for f in os.listdir(path):
        if string.find(f,".cam") <> -1:
            try:
                newcam = CalibCam()
                newcam.name=string.replace(f,".cam","") # the name of the camera is same as the file, removing the cam extension
                newcam.name=string.strip(newcam.name) # removing white chars
                fp = open(path+"/"+f,'r') # open the found file in folder
                print "Parsing : " +path+"/"+f
                line=fp.readline() # read the first line
                while line <> "": # continue while not EOF
                    commentstarts=string.find(line,"#") # find a comment tag
                    if commentstarts == -1: # if not found
                        commentstarts=len(line)-1 # set it to the end od string
                    sep=string.split( line[ :commentstarts],"=") # split the values up to the comment
                    if len(sep) > 1: # if there is at least the attribure and value
                        attr=sep[0] # name of attribute
                        value=sep[1] # value of attribute
                        value=string.strip( string.replace( value, ";",""))
                        if attr == "C": # if attribute is camera Centre
                            loc=string.split(value,',') # split x,y,z values
                            newcam.location[0]=float( loc[0]) # set the camera x position
                            newcam.location[1]=float( loc[1]) # set the camera y position
                            newcam.location[2]=float( loc[2]) # set the camera z position
                        if attr == "f": # if attribute is camera Centre
                            newcam.lens=float( value)*16 # set the camera lens - due to blender the focal length has to be multiplied by 16, blender has his own representation
                        if attr == "R": # if attribute is camera rotation
                            rot=string.split(value,',') # split x,y,z values
                            newcam.rotation[0]=radians(float( rot[0])) # set the camera x rotation
                            newcam.rotation[1]=radians(float( rot[1])) # set the camera y rotation
                            newcam.rotation[2]=radians(float( rot[2])) # set the camera z rotation
                        if attr == "k": # if attribute is render aspect ratio
                            asp=string.split(value,':') # split x,y values
                            newcam.aspx=int( asp[0]) # set the camera aspect x ratio
                            newcam.aspy=int( asp[1]) # set the camera aspect y ratio
                        if attr == "size": # if attribute is output image size
                            sz=string.split(value,'x') # split x,y values
                            newcam.sizex=int( sz[0]) # set the image size x 
                            newcam.sizey=int( sz[1]) # set the image size y 
                        if attr == "format": # if attribute is format of output image
                            newcam.format=string.strip(value) # output format type and strip white chars
                    line=fp.readline() # read next line if EOF, readline returns empty string
                # append a new camera to list
                print "Camera %s\n - location: %f,%f,%f" % (newcam.name,newcam.location[0],newcam.location[1],newcam.location[2])
                print " - rotation: %f,%f,%f" % (newcam.rotation[0],newcam.rotation[1],newcam.rotation[2])
                print " - aspect ratio %d:%d" % (newcam.aspx,newcam.aspy)
                print " - lens (~focal length * 16): %f" % (newcam.lens)
                print " - image size : %dx%d" % (newcam.sizex,newcam.sizey)
                print " - image format : " + newcam.format
                cameras.append(newcam)
                fp.close()
            except:
                print "!! Exception during parsing the "+path+"/"+f+" file"
    return cameras

# This function parse the xml.cam file and adds cameras into list of cameras
def loadCamera( file):
    """
    def loadCamera( file): returns rahat.camera object.
    file is the *.cam ASCII file with camera definitions.
    
    file example:
     C=10.0,0.3,-0.0; # camera centre x,y,z
     f=1; # focal plane
     R=10.0,35.3,-60; # camera rotation around x,y,z
     k=4:3;    # ascpect ration u:v
    
     # rendering settings
     size=800x600 # resolution in pixels
     format=tga # output format
     ##    * aviraw: Uncompressed AVI files. AVI is a commonly used format on Windows plattforms
     ##    * avijpeg: AVI movie w/ Jpeg images
     ##    * avicodec: AVI using win32 codec
     ##    * quicktime: Quicktime movie (if enabled)
     ##    * tga: Targa files
     ##    * rawtga: Raw Targa files
     ##    * png: Png files
     ##    * bmp: Bitmap files
     ##    * jpg: Jpeg files
     ##    * hamx: Hamx files
     ##    * iris: Iris files
     ##    * iriz: Iris + z-buffer files
     ##    * ftype: Ftype file
    """
    newcam = None
    try:
        newcam = CalibCam()
        start = 0
        if string.rfind(file,"/") < 0:
                    start = string.rfind( file, "\\")
        else: # Unix Windows Compactibility
            start = string.rfind(file,"/")
        newcam.name=string.replace( file[ start+1 :  ], ".cam", "") # the name of the camera is same as the file, removing the cam extension
        newcam.name=string.strip( newcam.name) # removing white chars
        fp = open( file, 'r') # open the found file in folder
        line=fp.readline() # read the first line
        while line <> "": # continue while not EOF
            commentstarts=string.find(line,"#") # find a comment tag
            if commentstarts == -1: # if not found
                commentstarts=len(line)-1 # set it to the end od string
            sep=string.split( line[ :commentstarts],"=") # split the values up to the comment
            if len(sep) > 1: # if there is at least the attribure and value
                attr=sep[0] # name of attribute
                value=sep[1] # value of attribute
                value=string.strip( string.replace( value, ";",""))
                #print attr,value
                if attr == "C": # if attribute is camera Centre
                    loc=string.split(value,',') # split x,y,z values
                    newcam.location[0]=float( loc[0]) # set the camera x position
                    newcam.location[1]=float( loc[1]) # set the camera y position
                    newcam.location[2]=float( loc[2]) # set the camera z position
                if attr == "f": # if attribute is camera Centre
                    newcam.lens=float( value)*16 # set the camera lens - due to blender the focal length has to be multiplied by 16, blender has his own representation
                if attr == "R": # if attribute is camera rotation
                    rot=string.split(value,',') # split x,y,z values
                    newcam.rotation[0]=radians(float( rot[0])) # set the camera x rotation
                    newcam.rotation[1]=radians(float( rot[1])) # set the camera y rotation
                    newcam.rotation[2]=radians(float( rot[2])) # set the camera z rotation
                if attr == "k": # if attribute is render aspect ratio
                    asp=string.split(value,':') # split x,y values
                    newcam.aspx=int( asp[0]) # set the camera aspect x ratio
                    newcam.aspy=int( asp[1]) # set the camera aspect y ratio
                if attr == "size": # if attribute is output image size
                    sz=string.split(value,'x') # split x,y values
                    newcam.sizex=int( sz[0]) # set the image size x 
                    newcam.sizey=int( sz[1]) # set the image size y 
                if attr == "format": # if attribute is format of output image
                    newcam.format=string.strip(value) # output format type and strip white chars
            line=fp.readline() # read next line if EOF, readline returns empty string
        # append a new camera to list
        print "Camera %s\n - location: %f,%f,%f" % (newcam.name,newcam.location[0],newcam.location[1],newcam.location[2])
        print " - rotation: %f,%f,%f" % (newcam.rotation[0],newcam.rotation[1],newcam.rotation[2])
        print " - aspect ratio %d:%d" % (newcam.aspx,newcam.aspy)
        print " - lens (~focal length * 16): %f" % (newcam.lens)
        print " - image size : %dx%d" % (newcam.sizex,newcam.sizey)
        print " - image format : " + newcam.format        
        fp.close()
    except:
        print "!! Exception during parsing the "+file+" file"
    return newcam
    
def saveCamera( camera, file):
    """
    def saveCamera( camera, path): writes rahat.camera object to the file.
    path is the path where should be file saved. File name is camera_name.cam
    file is the *.cam ASCII file with camera definitions.
    
    file example:
     C=10.0,0.3,-0.0; # camera centre x,y,z
     f=1; # focal plane
     R=10.0,35.3,-60; # camera rotation around x,y,z
     k=4:3;    # ascpect ration u:v
    
     # rendering settings
     size=800x600 # resolution in pixels
     format=tga # output format
     ##    * aviraw: Uncompressed AVI files. AVI is a commonly used format on Windows plattforms
     ##    * avijpeg: AVI movie w/ Jpeg images
     ##    * avicodec: AVI using win32 codec
     ##    * quicktime: Quicktime movie (if enabled)
     ##    * tga: Targa files
     ##    * rawtga: Raw Targa files
     ##    * png: Png files
     ##    * bmp: Bitmap files
     ##    * jpg: Jpeg files
     ##    * hamx: Hamx files
     ##    * iris: Iris files
     ##    * iriz: Iris + z-buffer files
     ##    * ftype: Ftype file
    """    
    try:
        f=open(file,'w')
        f.write("C=%.3f,%.3f,%.3f;\n" % (camera.location[0],camera.location[1],camera.location[2]) )              
        f.write("R=%.3f,%.3f,%.3f;\n" % (degrees(camera.rotation[0]),degrees(camera.rotation[1]),degrees(camera.rotation[2])) )              
        f.write("f=%.3f;\n" % (float(camera.lens)/16.0) )  
        f.write("k=%.0f:%.0f;\n" % (camera.aspx, camera.aspy) )  
        f.write("size=%.0fx%.0f;\n" % (camera.sizex, camera.sizey) )  
        f.write("format=%s;\n" % (camera.format) )                    
        f.close()
        print "Camera settings saved to: ",file
    except:
        print "!! Error during writing the camera file !!"

def writePcalMatrices(path,cameras):
    "Writes all matrices needed to create a camera projective matrix into files"
    for c in cameras:
        P=c.getP()
        try:
            f=open(path+"/"+c.name+".Pmat",'w')
            for row in P:
                f.write("%9.13f, %9.13f, %9.13f, %9.13f;\n" % (row[0],row[1],row[2],row[3]))              
            f.close()
        except:
            print "!! Cannot export P matrix !!"
            print "!! Cannot open " + path+"/"+c.name+".Pmat "+" file for writing!!"
            return None
        P=c.getK()        
        try:
            f=open(path+"/"+c.name+".Kmat",'w')
            for row in P:
                f.write("%9.13f, %9.13f, %9.13f;\n" % (row[0],row[1],row[2]))              
            f.close()
        except:
            print "!! Cannot export K calibration matrix !!"
            print "!! Cannot open " + path+"/"+c.name+".Kmat "+" file for writing!!"
            return None
        P=c.getR()
        try:
            f=open(path+"/"+c.name+".Rmat",'w')
            for row in P:
                f.write("%9.13f, %9.13f, %9.13f;\n" % (row[0],row[1],row[2]))              
            f.close()
        except:
            print "!! Cannot export R rotation matrix !!"
            print "!! Cannot open " + path+"/"+c.name+".Rmat "+" file for writing!!"
            return None

def writePMatrix( file, camera):
    """
    def writePMatrix(file,camera): writes projective camera matrix to ASCII file
    """
    P=camera.getP()
    try:
        f=open(file,'w')
        for row in P:
            f.write("%9.13f, %9.13f, %9.13f, %9.13f;\n" % (row[0],row[1],row[2],row[3]))              
        f.close()
    except:
        print "!! Cannot export P matrix !!"
        print "!! Cannot open " + path+"/"+c.name+".Pmat "+" file for writing!!"
        return None
        