# This function renders polygon into matrix 
import numpy
from scipy import *
from types import *
from rahat.mymatrix import *

# Returns [x,y,z] list of points of the polygon
def getPolyCoords(P,vertices, bBox = None):
    # Check propper arguments
    if (len(vertices) < 3):
        print "getPolyCoords(vertices): Wrong argument!"
        return None
    # Project vertices
    v = list();
    for ver in vertices:
        v.append( projectVertex( P, ver))
    # Get their UV image coordinates
    uv = list();
    for ver in v:
        uv.append( getUV( ver))
    # Find bounding box
    left = uv[0][0]
    right = left
    top = right
    bottom = top
    if bBox == None:
        for u in uv:
            if u[0] < left: left=u[0]
            if u[0] > right: right=u[0]
            if u[1] < bottom: bottom=u[1]
            if u[1] > top: top=u[1]    
    else:
        left = bBox[0]
        right = bBox[1]
        bottom = bBox[2]
        top = bBox[3]
    # Sort vertices in descending order in y axis
    if uv[0][1] < uv[2][1]:
        tmp=uv[2]
        uv[2]=uv[0]
        uv[0]=tmp
    if uv[0][1] < uv[1][1]:
        tmp=uv[1]
        uv[1]=uv[0]
        uv[0]=tmp
    if uv[2][1] > uv[1][1]:
        tmp=uv[1]
        uv[1]=uv[2]
        uv[2]=tmp         
    # Edges uv[0]-uv[1], uv[1]-uv[2], uv[2]-uv[0] params:
    c0 = uv[0][0]*uv[1][1]-uv[1][0]*uv[0][1]
    b0 = uv[1][0]-uv[0][0]
    a0 = uv[0][1]-uv[1][1]
    c1 = uv[1][0]*uv[2][1]-uv[2][0]*uv[1][1]
    b1 = uv[2][0]-uv[1][0]
    a1 = uv[1][1]-uv[2][1]
    c2 = uv[2][0]*uv[0][1]-uv[0][0]*uv[2][1]
    b2 = uv[0][0]-uv[2][0]
    a2 = uv[2][1]-uv[0][1]
    # Check if are positive for corners otherwise switch
    if (a0*uv[2][0] + b0*uv[2][1] + c0) < 0:
        a0=-a0
        b0=-b0
        c0=-c0
    if (a1*uv[0][0] + b1*uv[0][1] + c1) < 0:
        a1=-a1
        b1=-b1
        c1=-c1
    if (a2*uv[1][0] + b2*uv[1][1] + c2) < 0:
        a2=-a2
        b2=-b2
        c2=-c2
    # Get the projected plane parameters
    ver = list()
    ver.append( [ uv[0][0], uv[0][1], v[0][2]])
    ver.append( [ uv[1][0], uv[1][1], v[1][2]])
    ver.append( [ uv[2][0], uv[2][1], v[2][2]])    
    n = getPlaneParams( vertices)   
    # Proceed the drawing        
    coords = list();
    for y in range(int(bottom),int(top)):
        for x in range(int(left),int(right)):
            z = getZcoord( [x,y], n) 
            #z = 0
            e0 = a0*double(x) + b0*double(y) + c0
            e1 = a1*double(x) + b1*double(y) + c1
            e2 = a2*double(x) + b2*double(y) + c2                         
            if (e0 >= 0) and (e1 >= 0) and (e2 >= 0): 
                coords.append( [ x,y,z])
    return coords
    
    
    
# This returns image coordinates of the vertex [x,y,z]
# and returns its u,v coordinate in image and also the
# z distance
def projectVertex(P,vertex):
    homxy=mm(P,[[vertex[0]],[vertex[1]],[vertex[2]],[1.0]])
    return [homxy[0][0],homxy[1][0],homxy[2][0]]

# Returns UV coordinates from homogenous xy
def getUV(uvz):
    uv=[uvz[0]/uvz[2],uvz[1]/uvz[2]]
    return uv

# Returns plane parameters    
def getPlaneParams(vertices):
    # Theory:
    # The plane can be described as ax+by+cz+d=0
    # vhere the [a,b,c] is the normal vector of plane
    # and the is the shift
    V1=[ vertices[1][0]-vertices[0][0],
         vertices[1][1]-vertices[0][1],
         vertices[1][2]-vertices[0][2]]    
    V2=[ vertices[2][0]-vertices[0][0],
         vertices[2][1]-vertices[0][1],
         vertices[2][2]-vertices[0][2]]    
    n=cross(V1,V2)
    d = -(n[0]*vertices[0][0]+n[1]*vertices[0][1]+n[2]*vertices[0][2])   
    return [n[0],n[1],n[2],d]

def getZcoord(uv,n):
    # The n is a row vector describing the plane
    # uv are coordinates in 2D    
    # converting to matrix 
    z = -(n[0]*uv[0]+n[1]*uv[1]+n[3])/n[2]
    return z

# This function renders polygon into Matrix with 
# Polygon index and z coordinate
def draw3DPoly(vertices, Pmatrix, matrix, id):
    # Proper arguments test
    #if type(vertices) != Types.MeshType:
    #    print "Wrong argument! - renderObj(BlenderObject, Pmatrix, width, height)"
    #    print "                  BlenderObject must be MeshType!"
    #    return None
 
    # First found bounding box
    uv=getUV( projectVertex( Pmatrix, vertices[0]) )
    left=uv[0]
    right=uv[0]
    top=uv[0]
    bottom=uv[0]        
    for v in vertices:
        # note that C is a Matrix Object
        # Also draw a point to the Map
        uv = getUV ( projectVertex( Pmatrix, v) )
        #print uv
        if uv[0] < left: left=uv[0]
        if uv[0] > right: right=uv[0]
        if uv[1] < bottom: bottom=uv[1]
        if uv[1] > top: top=uv[1]    
    # Chceck proper coordinates
    if (left >= size(matrix[0],0)): return
    if (bottom >= size(matrix[0],1)): return
    if (right < 0): return
    if (top < 0): return
    # Check if the bounding box is inside the matrix
    if (left < 0): left = 0
    if (bottom < 0): bottom = 0
    if (top >= size(matrix[0],1)): top = size(matrix[0],1) -1
    if (right >= size(matrix[0],0)): right = size(matrix[0],0) -1
    # Get the list of coordinates
    coords = getPolyCoords( Pmatrix, vertices,[left,right,bottom,top])
    print [left,right,bottom,top]
    # Proceed the drawing    
    for c in coords:
        x = c[0]
        y = c[1]
        z = abs(c[2])        
        if (matrix[1][x,y] > z) or (matrix[1][x,y] == 0):
            matrix[0][x,y]=id
            matrix[1][x,y]=z
   